import time

from contextlib import contextmanager


class AureliaDriver:
    # From https://github.com/aurelia/tools/blob/master/plugins/protractor.js#L22
    AURELIA_STARTED_SCRIPT = '''
var cb = arguments[arguments.length - 1];
if (window.webpackJsonp && document.querySelector("[aurelia-app]")) { cb("Aurelia composed") }
document.addEventListener("aurelia-composed", function (e) {
    cb("Aurelia App composed")
}, false);
    '''
    AURELIA_STARTED_CONDITION_TEMPLATE = '''
if ({condition}) {{ cb("Aurelia composed") }}
    '''
    # From https://github.com/aurelia/tools/blob/master/plugins/protractor.js#L34
    AURELIA_NAVIGATE_COMPLETE = '''
var cb = arguments[arguments.length - 1];
document.querySelector("[aurelia-app]")
    .aurelia.subscribeOnce("router:navigation:complete", function() {
        cb(true)
    });
    '''

    def __init__(
        self,
        selenium_webdriver,
        script_timeout=10,
        default_wait_time=2,
        started_condition=None
    ):
        self.selenium = selenium_webdriver
        self.script_timeout = script_timeout
        self.started_condition = started_condition
        self.default_wait_time = default_wait_time

    def load_url(self, url):
        self.selenium.get(url)
        self.selenium.set_script_timeout(self.script_timeout)
        script = self.AURELIA_STARTED_SCRIPT
        if self.started_condition:
            script += self.AURELIA_STARTED_CONDITION_TEMPLATE.format(
                condition=self.started_condition.strip()
            )
        self.selenium.execute_async_script(script)
        time.sleep(self.default_wait_time)

    def wait(self, seconds=0):
        if seconds <= 0:
            seconds = self.default_wait_time
        time.sleep(seconds)

    @contextmanager
    def navigate(self):
        url_before_navigate = self.selenium.current_url
        yield
        if self.selenium.current_url == url_before_navigate:
            self.selenium.execute_async_script(self.AURELIA_NAVIGATE_COMPLETE)

    def find_element_by_binding(self, attr, value, bind_type='bind'):
        selector = self._get_binding_selector(attr, value, bind_type)
        return self.selenium.find_element_by_css_selector(selector)

    def _get_binding_selector(self, attr, value, bind_type):
        return '[{attr}\\.{bind_type}="{value}"]'.format(
            attr=attr,
            value=value,
            bind_type=bind_type
        )

    def find_elements_by_binding(self, attr, value, bind_type='bind'):
        selector = self._get_binding_selector(attr, value, bind_type)
        return self.selenium.find_elements_by_css_selector(selector)

    def __getattr__(self, name):
        if hasattr(self.selenium, name):
            return getattr(self.selenium, name)
        else:
            msg = "Neither 'AureliaDriver' nor 'WebDriver' has a '{name}' attribute"
            msg = msg.format(name=name)
            raise AttributeError(msg)



import pytest

from selenium_aurelia import AureliaDriver
from unittest.mock import MagicMock


class DriverStub:
    current_url = 'http://localhost'


@pytest.fixture
def aurelia_driver():
    return AureliaDriver(MagicMock())


def test_load_url(mock, aurelia_driver):
    url = 'http://localhost/'
    sleep = MagicMock()
    mock.patch('selenium_aurelia.time.sleep', sleep)

    aurelia_driver.load_url(url)

    aurelia_driver.selenium.get.assert_called_once_with(url)
    aurelia_driver.selenium.execute_async_script.assert_called_once_with(
        aurelia_driver.AURELIA_STARTED_SCRIPT
    )
    sleep.assert_called_once_with(aurelia_driver.default_wait_time)


def test_load_url_with_started_condition(mock, aurelia_driver):
    url = 'http://localhost/'
    sleep = MagicMock()
    mock.patch('selenium_aurelia.time.sleep', sleep)
    aurelia_driver.started_condition = '''
document.querySelector("[aurelia-app]").children[0].getAttribute('class') === 'splash'
    '''
    expected_line_script = '''
if (document.querySelector("[aurelia-app]").children[0].getAttribute('class') === 'splash') { cb("Aurelia composed") }
    '''  # noqa

    aurelia_driver.load_url(url)

    aurelia_driver.selenium.get.assert_called_once_with(url)
    assert aurelia_driver.selenium.execute_async_script.called
    script = aurelia_driver.selenium.execute_async_script.call_args[0][0].strip()
    script_lines = script.split('\n')
    assert len(script_lines) == 7
    assert script_lines[-1] == expected_line_script.strip()
    sleep.assert_called_once_with(aurelia_driver.default_wait_time)


def test_default_wait(mock, aurelia_driver):
    sleep = MagicMock()
    mock.patch('selenium_aurelia.time.sleep', sleep)

    aurelia_driver.wait()

    sleep.assert_called_once_with(aurelia_driver.default_wait_time)


def test_wait(mock, aurelia_driver):
    sleep = MagicMock()
    mock.patch('selenium_aurelia.time.sleep', sleep)

    aurelia_driver.wait(42)

    sleep.assert_called_once_with(42)


def test_navigate(aurelia_driver):
    aurelia_driver.selenium.current_url = 'http://localhost/url1'
    with aurelia_driver.navigate():
        pass

    aurelia_driver.selenium.execute_async_script.assert_called_once_with(
        aurelia_driver.AURELIA_NAVIGATE_COMPLETE
    )


def test_navigate_immediate_load(aurelia_driver):
    aurelia_driver.selenium.current_url = 'http://localhost/url1'
    with aurelia_driver.navigate():
        aurelia_driver.selenium.current_url = 'http://localhost/url2'

    assert not aurelia_driver.selenium.execute_async_script.called


def test_get_selenium_driver_attribute(aurelia_driver):
    aurelia_driver.selenium = DriverStub()

    url = aurelia_driver.current_url

    assert url == 'http://localhost'


def test_get_inexistant_attribute(aurelia_driver):
    aurelia_driver.selenium = DriverStub()

    with pytest.raises(AttributeError) as e:
        aurelia_driver.yolo

    assert "Neither 'AureliaDriver' nor 'WebDriver' has a 'yolo' attribute" in str(e)


def test_find_element_by_binding(aurelia_driver):
    aurelia_driver.find_element_by_binding('src', 'heroSrc')
    aurelia_driver.selenium.find_element_by_css_selector.assert_called_once_with(
        '[src\\.bind="heroSrc"]'
    )


def test_find_element_by_binding_with_bind_type(aurelia_driver):
    aurelia_driver.find_element_by_binding('src', 'heroSrc', bind_type='one-way')
    aurelia_driver.selenium.find_element_by_css_selector.assert_called_once_with(
        '[src\\.one-way="heroSrc"]'
    )


def test_find_elements_by_binding(aurelia_driver):
    aurelia_driver.find_elements_by_binding('src', 'heroSrc')
    aurelia_driver.selenium.find_elements_by_css_selector.assert_called_once_with(
        '[src\\.bind="heroSrc"]'
    )


def test_find_elements_by_binding_with_bind_type(aurelia_driver):
    aurelia_driver.find_elements_by_binding('src', 'heroSrc', bind_type='one-way')
    aurelia_driver.selenium.find_elements_by_css_selector.assert_called_once_with(
        '[src\\.one-way="heroSrc"]'
    )



#!/usr/bin/env python3

import os
import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


here = os.path.abspath(os.path.dirname(__file__))


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = [
            'selenium_aurelia/test',
        ]
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


def read(*parts):
    return open(os.path.join(here, *parts), 'r').read()


install_requires = read('requires.txt').split('\n')
tests_requires = read('tests_requires.txt').split('\n')
long_description = read('README.rst')


setup(
    name='selenium-aurelia',
    version='0.1.0',
    packages=find_packages(),
    install_requires=install_requires,
    tests_require=tests_requires,
    cmdclass={'test': PyTest},
    author='Julien Enselme',
    author_email='julien.enselme@centrale-marseille.fr',
    description='Wrapper around selenium driver to ease testing of Aurelia based applications.',
    long_description=long_description,
    keywords='selenium aurelia',
    url='https://framagit.org/Jenselme/selenium-aurelia',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'License :: OSI Approved :: MIT License',
        'Topic :: Internet :: WWW/HTTP :: Browsers',
        'Topic :: Software Development :: Testing',
        'Topic :: Utilities',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



from selenium import webdriver
from selenium_aurelia import AureliaDriver


def run_example(url, selenium_driver):
    # We create the AureliaDriver from the selenium driver.
    # We specify a complementary started condition: it is possible that your app will be loaded
    # before the AureliaDriver has the time to register the proper callback in the browser.
    # In this case, load_url will fail because of a TimeOut. To prevent this, you can specify
    # a custom condition you know is fulfilled when Aurelia is started on your application.
    # Here, we check for the spinner: if it is here, the app is loading. If it is not, the app is
    # loaded. This help reduce failures for wrong reasons.
    started_condition = '''
document.querySelector("[aurelia-app]").children[0].getAttribute('class') === 'splash'
    '''
    driver = AureliaDriver(selenium_driver, started_condition=started_condition)

    # The load url method will open the requested url in the browser and wait for Aurelia to start
    driver.load_url(url)

    # You can access selenium attribute directly with the selenium property
    assert driver.selenium.current_url == url

    # But you can also access them directly on the driver
    create_form = driver.find_element_by_id('select-heroes-form')
    name_input = create_form.find_element_by_css_selector('input')
    name_input.send_keys('Player 1')
    submit_btn = create_form.find_element_by_css_selector('button')

    # If an action will cause a navigation, call inside the navigate context manager. It will
    # ensure that the navigation is done before continuing
    with driver.navigate():
        submit_btn.click()

    # You can wait with the wait method.
    driver.wait(3)

    # You can easily get element(s) by their binding, eg src.bind="heroSrc"
    driver.find_element_by_binding('src', 'heroSrc')
    assert len(driver.find_elements_by_binding('src', 'heroSrc')) >= 1

    driver.quit()


if __name__ == '__main__':
    # Go here: http://selenium-python.readthedocs.io/api.html for the list of drivers
    run_example('https://www.arenaoftitans.com/game/create', webdriver.Firefox())
